import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;




public class Parse {
	static String fileToParse = "wpi_v1.1.schedb";
	static String outputName = "schedb.json";

	public static void main(String[] args){
		if(args.length > 0){
			fileToParse = args[0];
		}
		if(args.length > 1){
			outputName = args[1];
		}
		
		ScheduleDB db = parseXML();
		outputJSON(db);
		System.out.println("Successfully converted " + fileToParse + "to " + outputName);
	}



	private static void outputJSON(ScheduleDB db) {
		Gson gson = new Gson();
		String json = gson.toJson(db, new TypeToken<ScheduleDB>(){}.getType());
		FileWriter out;
		try {
			out = new FileWriter(outputName);
			out.write(json);
			out.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}


	}

	public static ScheduleDB parseXML(){
		ScheduleDB parsedDB = new ScheduleDB();

		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		Document dom = null;
		try {
			DocumentBuilder db = dbf.newDocumentBuilder();
			dom = db.parse(fileToParse);
		} catch (SAXException e) {
			e.printStackTrace();
			System.exit(-1);
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
			System.exit(-1);
		}

		Element root = dom.getDocumentElement();


		parseSchedb(root, parsedDB);

	


		return parsedDB;
	}



	private static void parseSchedb(Element item, ScheduleDB parsedDB) {
		parsedDB.generated = item.getAttribute("generated");
		parsedDB.minutesPerBlock = Integer.parseInt(item.getAttribute("minutes-per-block"));
		NodeList deptList = item.getElementsByTagName("dept");

		for(int i = 0; i < deptList.getLength(); i++){
			parsedDB.departments.add(parseDept((Element)deptList.item(i), parsedDB));
		}

	}


	private static Department parseDept(Element item, ScheduleDB parsedDB) {
		Department d = new Department();
		d.abbrev = item.getAttribute("abbrev");
		d.name = fixCase(item.getAttribute("name"));
		NodeList courseList = item.getElementsByTagName("course");
		for(int i = 0; i < courseList.getLength(); i++){
			d.courses.add(parseCourse((Element) courseList.item(i), d));
		}
		return d;

	}



	private static Course parseCourse(Element item, Department d) {
		Course c = new Course();
		c.number = item.getAttribute("number");
		c.name = fixCase(item.getAttribute("name"));
		c.minCredits = item.getAttribute("min-credits");
		c.maxCredits = item.getAttribute("max-credits");
		c.gradeType = item.getAttribute("grade-type");
		c.note = item.getAttribute("note");
		c.cross_list = item.getAttribute("cross_list");
		c.description = item.getAttribute("course_desc");
		NodeList sectionList = item.getElementsByTagName("section");
		for(int i = 0; i < sectionList.getLength(); i++){
			c.sections.add(parseSection((Element) sectionList.item(i), c));
		}

		return c;

	}


	private static Section parseSection(Element item, Course c) {
		Section s = new Section();
		s.crn = item.getAttribute("crn");
		s.number = item.getAttribute("number");
		s.seats = item.getAttribute("seats");
		s.availableseats = item.getAttribute("availableseats");
		s.term = item.getAttribute("term");
		s.partOfTerm = item.getAttribute("part-of-term");
		s.note = item.getAttribute("note");
		s.max_waitlist = item.getAttribute("max_waitlist");
		s.actual_waitlist = item.getAttribute("actual_waitlist");
		NodeList periodList = item.getElementsByTagName("period");
		for(int i = 0; i < periodList.getLength(); i++){
			s.periods.add(parsePeriod((Element) periodList.item(i), s));
		}

		return s;
	}



	private static Period parsePeriod(Element item, Section s) {
		Period p = new Period();
		p.type = item.getAttribute("type");
		p.professor = item.getAttribute("professor");
		p.days = item.getAttribute("days");
		p.starts = parseTime(item.getAttribute("starts"));
		p.ends = parseTime(item.getAttribute("ends"));
		p.location = item.getAttribute("building") + item.getAttribute("room");
		p.professor_sort_name = item.getAttribute("professor_sort_name");
		p.professor_email = item.getAttribute("professor_email");

		return p;
	}







	public static String fixCase(String input){
		// Break it into words
		String strings[] = input.split("\\s+");
		for (int i = 0; i < strings.length; i++){
			// If its I, II, etc, just leave it uppercase
			if(!strings[i].matches("^M{0,4}(CM|CD|D?C{0,3})(XC|XL|L?X{0,3})(IX|IV|V?I{0,3})$")){
				// Make it all lowercase
				strings[i] = strings[i].toLowerCase();
				// If it should be capitalized, capitalize it
				if(shouldBeCapitalized(strings[i])){
					if(strings[i].length() > 1)
						strings[i] = Character.toUpperCase(strings[i].charAt(0)) + strings[i].substring(1);
					else
						strings[i] = Character.toUpperCase(strings[i].charAt(0)) + "";
				}
			}


		}
		String returnstring = strings[0];
		for(int i = 1; i < strings.length; i++){
			returnstring = returnstring + " " + strings[i];
		}
		return returnstring;
	}


	public static boolean shouldBeCapitalized(String s){
		if(s.compareTo("a") == 0 || s.compareTo("and") == 0 || s.compareTo("the") == 0 || s.compareTo("in") == 0 || s.compareTo("an") == 0 || s.compareTo("or") == 0 || s.compareTo("at") == 0 || s.compareTo("of") == 0)
			return false;
		return true;
	}

	public static int parseTime(String time){
		int ret = 0;
		try{
		if(time.length() != 0){
			ret += Integer.parseInt(time.substring(0, time.indexOf(':'))) * 100;
			ret += Integer.parseInt(time.substring(time.indexOf(':') + 1, time.indexOf(':') + 3));
			if(time.contains("PM")){
				ret += 1200;
			}
			if(ret >= 2400){
				ret -= 2400;
			}
		}
		}catch (Exception e){
			e.printStackTrace();
			return 0;
		}

		return ret;


	}




}
